package com.example.wei.mybluetooth;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Switch;
import android.widget.TextView;
import android.widget.Toast;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;

public class MainActivity extends Activity {

    private Switch bluetoothSwitch;
    private Button bluetoothSearch;
    private Spinner bluetoothList;
    private Button bluetoothConnect;
    private TextView recvData;
    private EditText sendData;
    private Button bluetoothSend;

    private Button upSend;
    private Button leftSend;
    private Button downSend;
    private Button rightSend;

    private BluetoothAdapter bluetoothAdapter;
    private List<String> list = new ArrayList<String>();
    private ArrayAdapter<String> adapter;
    private String strMacAddress;
    private boolean booleanConnect = false;
    private ConnectedThread connectedThread;
    private MyHandler myHandler = new MyHandler(this);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        bluetoothSwitch = (Switch) findViewById(R.id.swtch);
        bluetoothSearch = (Button) findViewById(R.id.search);
        bluetoothList = (Spinner) findViewById(R.id.list);
        bluetoothConnect = (Button) findViewById(R.id.connect);
        recvData = (TextView) findViewById(R.id.recvdata);
        sendData = (EditText) findViewById(R.id.senddata);
        bluetoothSend = (Button) findViewById(R.id.send);

        upSend = (Button)findViewById(R.id.BtnUp);
        leftSend = (Button)findViewById(R.id.BtnLeft);
        downSend = (Button)findViewById(R.id.BtnDown);
        rightSend = (Button)findViewById(R.id.BtnRight);

        bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if(bluetoothAdapter == null){
            //表明此手机不支持蓝牙
            Toast.makeText(MainActivity.this, "未发现蓝牙设备", Toast.LENGTH_SHORT).show();
            return;
        }

        if (bluetoothAdapter.isEnabled()) {
            bluetoothSwitch.setChecked(true);
        }

        adapter = new ArrayAdapter<String>(this,android.R.layout.simple_spinner_item, list);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        bluetoothList.setAdapter(adapter);

        bluetoothList.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> adapterView, View view, int i, long l) {
                strMacAddress = adapter.getItem(i);
                adapterView.setVisibility(View.VISIBLE);
            }

            @Override
            public void onNothingSelected(AdapterView<?> adapterView) {

            }
        });

        bluetoothSwitch.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if (b) {
                    if (!bluetoothAdapter.isEnabled()) { //蓝牙未开启，则开启蓝牙
                        Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                        startActivity(enableIntent);
                    } else {
                        Toast.makeText(MainActivity.this, "蓝牙已开启", Toast.LENGTH_SHORT).show();
                    }
                } else {
                    bluetoothAdapter.disable();
                    Toast.makeText(MainActivity.this, "蓝牙已关闭", Toast.LENGTH_SHORT).show();
                }
            }
        });


        bluetoothSearch.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (bluetoothAdapter == null) {
                    Toast.makeText(MainActivity.this, "未发现蓝牙设备", Toast.LENGTH_SHORT).show();
                    return;
                } else if (!bluetoothAdapter.isEnabled()) {
                    Toast.makeText(MainActivity.this, "蓝牙设备未开启", Toast.LENGTH_SHORT).show();
                }

                Set<BluetoothDevice> pairedDevices = bluetoothAdapter.getBondedDevices();
                if (pairedDevices.size() > 0) {
                    for (BluetoothDevice device : pairedDevices) {
                        adapter.remove(device.getAddress());
                        adapter.add(device.getAddress());
                    }
                } else {
                    //注册，当一个设备被发现时调用mReceive
                    IntentFilter filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
                    registerReceiver(mReceiver, filter);
                }
            }
        });


        bluetoothConnect.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (strMacAddress == null) {
                    Toast.makeText(MainActivity.this, "请先搜索设备", Toast.LENGTH_SHORT).show();
                } else {
                    BluetoothDevice device = bluetoothAdapter.getRemoteDevice(strMacAddress);
                    ConnectThread connectThread = new ConnectThread(device);
                    connectThread.start();
                }
            }
        });


        bluetoothSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (booleanConnect == false) {
                    Toast.makeText(MainActivity.this, "请先连接设备", Toast.LENGTH_SHORT).show();
                } else {
                    String strSendData = new String(sendData.getText().toString());
                    connectedThread.write(strSendData.getBytes());
                    sendData.setText("");
                }
            }
        });

        upSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (booleanConnect == false) {
                    Toast.makeText(MainActivity.this, "请先连接设备", Toast.LENGTH_SHORT).show();
                } else {
                    String str = "ONA\n";
                    String strSendData = new String(str);
                    connectedThread.write(strSendData.getBytes());
                    sendData.setText("");
                }
            }
        });

        downSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (booleanConnect == false) {
                    Toast.makeText(MainActivity.this, "请先连接设备", Toast.LENGTH_SHORT).show();
                } else {
                    String str = "ONB\n";
                    String strSendData = new String(str);
                    connectedThread.write(strSendData.getBytes());
                    sendData.setText("");
                }
            }
        });

        leftSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (booleanConnect == false) {
                    Toast.makeText(MainActivity.this, "请先连接设备", Toast.LENGTH_SHORT).show();
                } else {
                    String str = "ONC\n";
                    String strSendData = new String(str);
                    connectedThread.write(strSendData.getBytes());
                    sendData.setText("");
                }
            }
        });

        rightSend.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (booleanConnect == false) {
                    Toast.makeText(MainActivity.this, "请先连接设备", Toast.LENGTH_SHORT).show();
                } else {
                    String str = "OND\n";
                    String strSendData = new String(str);
                    connectedThread.write(strSendData.getBytes());
                    sendData.setText("");
                }
            }
        });
    }
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

    private static class MyHandler extends Handler {
        WeakReference<MainActivity> mActivity;

        MyHandler(MainActivity activity) {
            mActivity = new WeakReference<MainActivity>(activity);
        }

        @Override
        public void handleMessage(Message msg) {
            MainActivity theActivity = mActivity.get();

            theActivity.recvData.append(msg.obj.toString());
        }
    }

    @Override
    protected void onDestroy() {
        if (connectedThread != null) {
            connectedThread.cancel();
        }
        super.onDestroy();
    }


    private BroadcastReceiver mReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if(BluetoothDevice.ACTION_FOUND.equals(action)){
                BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                // 已经配对的则跳过
                if (device.getBondState() != BluetoothDevice.BOND_BONDED) {
                    adapter.add(device.getAddress());
                }
            }else if (BluetoothAdapter.ACTION_DISCOVERY_FINISHED.equals(action)) {  //搜索结束
                if (adapter.getCount() == 0) {
                    Toast.makeText(MainActivity.this, "没有搜索到设备", Toast.LENGTH_SHORT).show();
                }
            }
        }
    };



    private class ConnectThread extends Thread{
        private BluetoothSocket mmsocket;
        private BluetoothDevice mmdevice;

        public ConnectThread(BluetoothDevice device){
            mmdevice = device;
            BluetoothSocket tmp = null;
            try {
                tmp = mmdevice.createRfcommSocketToServiceRecord(UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"));
            } catch (IOException e) {

            }
            mmsocket = tmp;
        }

        public void run(){
            bluetoothAdapter.cancelDiscovery();    //取消设备查找
            try {
                mmsocket.connect();
            } catch (IOException e) {
                try {
                    mmsocket.close();
                } catch (IOException e1) {

                }
                //连接失败
                return;
            }


            booleanConnect = true;
            //新建一个数据交换线程
            connectedThread = new ConnectedThread(mmsocket);
            connectedThread.start();
        }

        public void cancel() {
            try {
                mmsocket.close();
            } catch (IOException e) {

            }
        }
    }


    private class ConnectedThread extends Thread{
        private BluetoothSocket mmsocket;
        private InputStream inStream;
        private OutputStream outStream;

        public ConnectedThread(BluetoothSocket socket){

            mmsocket = socket;
            try {
                //获得输入输出流
                inStream = mmsocket.getInputStream();
                outStream = mmsocket.getOutputStream();
            } catch (IOException e) {

            }
        }

        public void run(){
            byte[] buff = new byte[1];
            int len = 0;
            //读数据需不断监听，写不需要
            while(true){
                try {
                    len = inStream.read(buff);
                    //把读取到的数据发送给UI进行显示
                    String strBuffer = new String(buff);

                    Message toMain = myHandler.obtainMessage();
                    toMain.obj = strBuffer;
                    myHandler.sendMessage(toMain);
                } catch (IOException e) {

                    break;
                }
            }
        }

        public void write(byte[] buffer) {
            try {
                outStream.write(buffer);
            } catch (IOException e) {

            }
        }

        public void cancel() {
            try {
                mmsocket.close();
            } catch (IOException e) {

            }
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            super.finish();
            return true;
        }

        return super.onOptionsItemSelected(item);
    }
}
